/**
  ******************************************************************************
  * @file    function.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   This file contains common functions used by the demo applications.
  ******************************************************************************
  * @attention
  *
  */ 

// Includes ------------------------------------------------------------------
#include "functions.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
uint32_t GTimeStamp;
uint16_t TimeOutStartTime;
uint8_t EEPBuffer[16];
uint8_t GUseEeprom;

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

/**
  * @brief  Initialize SPI.
  * @param  None
  * @retval None
  */ 
void Init_SPIFullDuplex(void)
{
  // Set the MOSI,MISO and SCK at high level
  GPIO_ExternalPullUpConfig(GPIOB, GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7, ENABLE);
  SPI_DeInit(SPI1);
  SPI_Init(SPI1, SPI_FirstBit_MSB, SPI_BaudRatePrescaler_4, SPI_Mode_Master, SPI_CPOL_High,
           SPI_CPHA_2Edge, SPI_Direction_2Lines_FullDuplex, SPI_NSS_Soft, 0x07);

  SPI_Cmd(SPI1, ENABLE);

  // Otherwise the LCD display doesn't work when the board is powered-up ... 
  WaitDelay(50);
}


/**
  * @brief  Initialize TIM2.
  * @param  None
  * @retval None
  */ 
void Init_TIM2(void)
{
  /* TimerTick = 1 ms
     Warning: fcpu must be equal to 16MHz
     fck_cnt = fck_psc/presc = fcpu/128 = 125kHz --> 1 tick every 8 s
     ==> 1 ms / 8 s = 125 ticks
   */
  TIM2_TimeBaseInit(TIM2_Prescaler_128, TIM2_CounterMode_Up, 125);
  TIM2_UpdateRequestConfig(TIM2_UpdateSource_Global);

  TIM2_ITConfig(TIM2_IT_Update, ENABLE);
  TIM2_Cmd(ENABLE);

}

/**
  * @brief  Initialize GPIOs.
  * @param  None
  * @retval None
  */ 
void Init_GPIO(void)
{
  // Initialize I/Os in Output Mode for LEDs 
  STM_EVAL_LEDInit(LED1);
  STM_EVAL_LEDInit(LED2);
  STM_EVAL_LEDInit(LED3);
  STM_EVAL_LEDInit(LED4);
  STM_EVAL_LEDInit(LED5);
}

/**
  * @brief  Initialize the device clock.
  * @param  None
  * @retval None
  */ 
void Init_Clock(void)
{
  CLK_PeripheralClockConfig(CLK_Peripheral_RTC,  ENABLE);
  CLK_PeripheralClockConfig(CLK_Peripheral_ADC1, ENABLE);
  CLK_PeripheralClockConfig(CLK_Peripheral_TIM2, ENABLE);
  CLK_PeripheralClockConfig(CLK_Peripheral_I2C1, ENABLE);
  CLK_PeripheralClockConfig(CLK_Peripheral_SPI1, ENABLE);
  CLK_PeripheralClockConfig(CLK_Peripheral_COMP, ENABLE);
  CLK_PeripheralClockConfig(CLK_Peripheral_DAC, ENABLE);
  CLK_PeripheralClockConfig(CLK_Peripheral_TIM1, ENABLE);
  CLK_PeripheralClockConfig(CLK_Peripheral_TIM4, ENABLE);

  // RTC Clock source Configuration
  CLK_RTCClockConfig(CLK_RTCCLKSource_LSE, CLK_RTCCLKDiv_1);

  // Select HSE as system clock source 
  CLK_SYSCLKSourceSwitchCmd(ENABLE);
  CLK_SYSCLKSourceConfig(CLK_SYSCLKSource_HSE);
  CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);
  while (CLK_GetSYSCLKSource() != CLK_SYSCLKSource_HSE)
  {}

  ITC_SetSoftwarePriority(RTC_IRQn, ITC_PriorityLevel_1);
  ITC_SetSoftwarePriority(TIM4_UPD_OVF_TRG_IRQn, ITC_PriorityLevel_2);
  ITC_SetSoftwarePriority(EXTI7_IRQn, ITC_PriorityLevel_3);


  ITC_SetSoftwarePriority(TIM1_UPD_OVF_TRG_IRQn, ITC_PriorityLevel_2);
  ITC_SetSoftwarePriority(TIM1_CC_IRQn, ITC_PriorityLevel_1);
  ITC_SetSoftwarePriority(TIM2_UPD_OVF_TRG_BRK_IRQn, ITC_PriorityLevel_0);
}

/**
  * @brief  Initialize the application.
  * @param  None
  * @retval None
  */  
void Init_Application(void)
{
  // Variables initialization 
  ButtonStatus = 0;

  // Enable interrupts 
  enableInterrupts();
  
  // Clock initialization 
  Init_Clock();

  // Initialize time application 
  Calendar_Init();

  // Clear the previosly alarm if occured 
  RTC_ClearFlag(RTC_FLAG_ALRAF);

  // I/O Port initialization 
  Init_GPIO();

  // TIMER2 initialization for generic timebase 
  Init_TIM2();

  // LCD initialization: Init_SPISimplex must be executed before 
  STM8_EVAL_LCD_Init();

  // Menu initialization 
  MenuInit();


}

/**
  * @brief  Get the Timer time stamp.
  * @param  None
  * @retval Timer time stamp
  */
uint32_t GetTimeStamp(void)
{
  return GTimeStamp;
}

/**
  * @brief  Initialize the start time for generating a time-out.
  * @param  None
  * @retval None
  */   
void GetTimeOutStartTime(void)
{
  sim();
  TimeOutStartTime = (unsigned int)( GetTimeStamp() );
  rim();
}

/**
  * @brief  Generate a time out after Delay.
  * @note   Before calling this function, the function GetTimeOutStartTime()
  *         must be called.  
  * @param  Delay: delay before time-out generation based on timer tick
  * @retval None
  */  
bool TimeOut(uint16_t Delay)
{
  uint16_t TimeStamp;
  uint16_t PeriodStamp;

  sim();
  TimeStamp = (unsigned int)(GetTimeStamp());
  rim();

  PeriodStamp = TimeStamp - TimeOutStartTime;

  if (PeriodStamp >= Delay)
  {
    return TRUE;
  }
  else
  {
    return FALSE;
  }

}

/**
  * @brief  Wait a delay
  * @param  Delay: delay based on timer Tick
  * @retval None
  */    
void WaitDelay(uint16_t Delay)
{

  uint16_t DelayPeriodStart;
  __IO uint16_t TimeStamp;
  uint16_t PeriodStamp;

  sim();
  DelayPeriodStart = (uint16_t)(GetTimeStamp());
  rim();

  while (1)
  {
    sim();
    TimeStamp = (uint16_t)(GetTimeStamp());
    rim();
    PeriodStamp = TimeStamp - DelayPeriodStart;
    if (PeriodStamp >= Delay)
    {
      return;
    }
  }

}

