/**
  ******************************************************************************
  * @file    joystick_button.c
  * @author  MCD Application Team
  * @version V1.2.1
  * @date    19-May-2011
  * @brief   This file contains basic functions to control the joystick button
  ******************************************************************************
  * @attention
  *
  */ 

// Includes ------------------------------------------------------------------
#include "joystick_button.h"
#include "functions.h"


// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------

#define JOYSTICK_DEBOUNCE_NB   3 /* Number of samples to validate a joystick event */
#define BUTTON_DEBOUNCE_NB     3 /* Number of samples to validate a button event */

// Following timings are based on TimerTick 
#define JOYSTICK_PERIOD        2 /* Defines the period to read the joystick */
#define BUTTON_PERIOD          2 /*  Defines the period to read the button */

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
uint8_t ButtonStatus;

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

/**
  * @brief  Read the joystick state
  * @param  None
  * @retval Joystick state
  */    
JOYState_TypeDef ReadJoystick(void)
{
  uint16_t TimeStamp;
  uint16_t PeriodStamp;
  uint8_t CurrentJoystickState;
  static uint16_t JoystickPeriodStart;
  static uint8_t PreviousJoystickState;
  static uint8_t JoystickDebounceNb;

  sim();
  TimeStamp = (uint16_t)(GetTimeStamp());
  rim();

  PeriodStamp = TimeStamp - JoystickPeriodStart;

  if (PeriodStamp >= JOYSTICK_PERIOD)
  {

    // Adjust TimeStamp in a Period range. 
    while (PeriodStamp >= JOYSTICK_PERIOD)
    {
      PeriodStamp -= JOYSTICK_PERIOD;
      JoystickPeriodStart += JOYSTICK_PERIOD;
    }

    /* Read PD[4..5], PD7 and PE[1..2]:
       PE1 = LEFT
       PE2 = SEL
       PD4 = DOWN
       PD5 = RIGHT
       PD7 = UP
       CurrentJoystickState:
       bit7 = JOY_UP
       bit5 = JOY_RIGHT
       bit4 = JOY_DOWN
       bit2 = JOY_SEL
       bit1 = JOY_LEFT
     */
    CurrentJoystickState = (uint8_t)(GPIO_ReadInputData(GPIOE) & (uint8_t)0x06);
    CurrentJoystickState |= (uint8_t)(GPIO_ReadInputData(GPIOD) & (uint8_t)0xB0);

    if (CurrentJoystickState != PreviousJoystickState)
    {
      PreviousJoystickState = CurrentJoystickState;
      JoystickDebounceNb = 0;
      SetBit(ButtonStatus, BUTTON_STATUS_JOYSTICK_DEBOUNCE_ENABLE);
    }
    else
    {
      if (ValBit(ButtonStatus, BUTTON_STATUS_JOYSTICK_DEBOUNCE_ENABLE))
      {

        JoystickDebounceNb++;

        if (JoystickDebounceNb >= (JOYSTICK_DEBOUNCE_NB - 1))
        {
          switch (CurrentJoystickState)
          {
            case 0x36:
              // Select button 
              CurrentJoystickState = JOY_UP;
              break;
            case 0x96:
              // Down button 
              CurrentJoystickState = JOY_RIGHT;
              break;
            case 0xA6:
              // Left button 
              CurrentJoystickState = JOY_DOWN;
              break;
            case 0xB2:
              // Right button 
              CurrentJoystickState = JOY_SEL;
              break;
            case 0xB4:
              // Up Button 
              CurrentJoystickState = JOY_LEFT;
              break;
            default:
              // Release Button 
              CurrentJoystickState = JOY_NONE;
              break;
          }
          ClrBit(ButtonStatus, BUTTON_STATUS_JOYSTICK_DEBOUNCE_ENABLE);
          return((JOYState_TypeDef)CurrentJoystickState);
        }
      }
    }
  }
  return(JOY_NONE);
}

/**
  * @brief  Read the Button key state
  * @param  None
  * @retval Button key state
  */     
BUTTON_ID_T ReadButton(void)
{
  static uint16_t ReadButtonPeriodStart;
  uint16_t TimeStamp;
  uint16_t PeriodStamp;
  uint8_t CurrentButtonState;
  static uint8_t PreviousButtonState;
  static uint8_t ButtonDebounceNb;

  sim();
  TimeStamp = (unsigned int)( GetTimeStamp() );
  rim();

  PeriodStamp = TimeStamp - ReadButtonPeriodStart;

  if ( PeriodStamp >= BUTTON_PERIOD )
  {

    while ( PeriodStamp >= BUTTON_PERIOD )
    { /* Adjust TimeStamp in a Period range */
      PeriodStamp -= BUTTON_PERIOD;
      ReadButtonPeriodStart += BUTTON_PERIOD;
    }

    // Read PC2 
    CurrentButtonState = (uint8_t)(GPIO_ReadInputData(GPIOC) & 0x02);

    if (CurrentButtonState != PreviousButtonState)
    {
      PreviousButtonState = CurrentButtonState;
      ButtonDebounceNb = 0;
      SetBit(ButtonStatus, BUTTON_STATUS_BUTTON_DEBOUNCE_ENABLE);
    }
    else
    {
      if (ValBit(ButtonStatus, BUTTON_STATUS_BUTTON_DEBOUNCE_ENABLE))
      {
        ButtonDebounceNb++;
        if (ButtonDebounceNb >= (BUTTON_DEBOUNCE_NB - 1))
        {
          switch (CurrentButtonState)
          {
            case 0x02:
              // Button released
              CurrentButtonState = B_RELEASED;
              break;
            default:
              // Button pressed
              CurrentButtonState = B_PRESSED;
              break;
          }
          ClrBit(ButtonStatus, BUTTON_STATUS_BUTTON_DEBOUNCE_ENABLE);
          return((BUTTON_ID_T)CurrentButtonState);
        }
      }
    }
  }
  return(B_NONE);
}

